/* Copyright (c) 2010 The Chromium OS Authors. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 */

#ifndef VBOOT_REFERENCE_CRC32_TEST_H_
#define VBOOT_REFERENCE_CRC32_TEST_H_

int TestCrc32TestVectors(void);

#endif  /* VBOOT_REFERENCE_CRC32_TEST_H_ */
