/* Copyright 2019 The Chromium OS Authors. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 *
 * This header may be imported to expose vboot SHA implementation.
 */

#ifndef VBOOT_REFERENCE_VB2_SHA_H_
#define VBOOT_REFERENCE_VB2_SHA_H_

#include "../2lib/include/2sha.h"

#endif  /* VBOOT_REFERENCE_VB2_SHA_H_ */
